# print.null_distribution / prints method, permutations, and observations

    Code
      res <- print(nd)
    Message
      
      -- Null Distribution 
      * Method: moran
      * Permutations: 10
      * Observations: 5

