# print.neuromaps_enhanced_comparison / prints without null model info when no nulls used

    Code
      res <- print(result)
    Message
      
      -- Brain Map Comparison 
      Method: pearson
      r = 0.5, p = 0.001
      n = 100

# print.neuromaps_enhanced_comparison / prints null model info when nulls used

    Code
      print(result)
    Message
      
      -- Brain Map Comparison 
      Method: pearson
      r = 0.5, p = 0.001
      n = 100
      --------------------------------------------------------------------------------
      Null model: burt2020 (100 permutations)
      p_null = 0.05

