% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transforms.R
\name{check_wb_command}
\alias{check_wb_command}
\title{Check for Connectome Workbench}
\usage{
check_wb_command(wb_path = NULL)
}
\arguments{
\item{wb_path}{Optional explicit path to \code{wb_command}.}
}
\value{
Path to \code{wb_command} executable.
}
\description{
Verifies that \code{wb_command} is available. If \code{wb_path} is \code{NULL},
checks \code{ciftiTools} default and system PATH.
}
\examples{
\dontrun{
check_wb_command()
}
}
